//blocks.h: A header file that defines the data structures and
//functions used to represent logic blocks and sensor blocks
//in the CROWBAR prototype. Implementation code can be found
//in Blocks.cc

#if !defined(_Blocks_h_)
#define _Blocks_h_

#include "signal.h"
#include "cell.h"

#define MAX_STRING 16

//We are not modeling the hysteresis of the optical receiver
//We are not modeling oscillations caused by poor programming
//--it is assumed that the outputs eventually converge

//We ARE modeling KNOWN bugs in the logic to match the prototype

///////////////////////////////////////////////////////////////////////////////
//For convenient passing of data between blocks:
typedef struct CLB_input_struct
{
  signal_type north, east, south, west;
  signal_type clk_a, clk_b; //two non-overlapping clocks
  signal_type reset_flop;
  signal_type config_enable;
  signal_type mux_config_in;
  signal_type lut_config_in;
  signal_type test_mode;
  signal_type scan_in; 
} CLB_input; 

typedef struct CSB_input_struct
{
  signal_type north, east, south, west; 
  signal_type clk_a, clk_b; //two non-overlapping clocks
  signal_type reset_flop;
  optical_type sensor;
  signal_type config_in;
  signal_type config_enable;
  signal_type config_opt;
  signal_type test_mode;
  signal_type scan_in; 
} CSB_input; 

///////////////////////////////////////////////////////////////////////////////

//Model of a Configurable Logic Block
class CLB_class {

public:
  
  //public data: nada

  //public functions
  CLB_class(); //constructor
  //CLB_class(unsigned int new_ID); //constructor
  ~CLB_class(); //destructor
  void Clear(); //erase all state, initialize all values to unknown
  void Update(CLB_input current_inputs);
  bool Converged(); //returns true if current and previous outputs match
  signal_type Get_Output(); //return output of cell
  signal_type Get_Test_Output(); //return output of cell
  signal_type Get_Lut_Config_Out(); //output of first config string
  signal_type Get_Mux_Config_Out(); //output of second config string
  signal_type Get_Lut_Config_Bit(unsigned int index); //get arb. bit
  signal_type Get_Mux0_Config_Bit(unsigned int index); //get arb. bit
  signal_type Get_Mux1_Config_Bit(unsigned int index); //get arb. bit
  signal_type Get_Mux2_Config_Bit(unsigned int index); //get arb. bit

protected:

  //protected structures
  LUT8 Lookup_table;
  configurable_mux4 Mux0, Mux1, Mux2;
  resettable_flop Flop;

private:

  //private data
  signal_type current_output; //current output of LUT
  signal_type previous_output;
  signal_type current_test_output;
  signal_type previous_test_output;
  //unsigned int ID;

}; //CLB_class

///////////////////////////////////////////////////////////////////////////////

//Model of a Configurable Sensor Block
class CSB_class {

public:
  
  //public data: nada

  //public functions
  CSB_class(); //constructor
  //CSB_class(unsigned int new_ID); //constructor
  ~CSB_class(); //destructor
  void Clear(); //erase all state, initialize all values to unknown
  void Update(CSB_input inputs);
  bool Converged();
  signal_type Get_Output(); //return Mux output of cell
  signal_type Get_Test_Output(); //return latched output of cell
  signal_type Get_Config_Out(); //return config-chain output of cell
  signal_type Get_Receiver_Config_Bit(unsigned int index); //get arbitrary bit
  signal_type Get_Mux_Config_Bit(unsigned int index); 

protected:

  //protected structures
  receiver Optical_Receiver;
  configurable_mux8 Mux;
  resettable_flop Flop;

private:

  //private data
  signal_type current_output;
  signal_type previous_output;
  signal_type current_test_output;
  signal_type previous_test_output;
  //unsigned int ID;

}; //CSB_class

///////////////////////////////////////////////////////////////////////////////

//Model of a pin/pad pair
//FIXME: implementation file out of date
enum pin_type { io, dedicated_input, dedicated_output, vdd, gnd };
enum pin_direction_type { input, output };

void Print_pin_type(pin_type type);

class Pin_class {
 public:
  //public functions
  Pin_class(); //default constructor
  ~Pin_class(); //destructor
  void Configure(pin_type new_type,
		 assertion_type new_assertion_level);
  void Clear(); //Clear variables
  void Update(); //effect depends on orientation
  void Update_Pad_Value(voltage_type voltage); //for input pins
  void Update_Internal_Value(signal_type signal); //for output pins
  void Update_Direction(pin_direction_type new_direction); 
  voltage_type Get_Pad_Value();
  signal_type Get_Internal_Value();
  signal_type Get_Inverted_Internal_Value();
  pin_direction_type Get_Direction();
  pin_type Get_Type();

 private:
  //private functions
  //private data
  bool configured; //only allow a pin to be configured once
  pin_type type;
  assertion_type assertion_level;
  pin_direction_type direction;
  voltage_type pad_value;
  signal_type internal_value;
}; //Pin_class

///////////////////////////////////////////////////////////////////////////////
#endif

//end of Blocks.h










